﻿using System.Linq;
using System.Web.Mvc;
using ExampleWebApplication.Entities;
using ExampleWebApplication.Filters;
using ExampleWebApplication.Infraestruture.Aggregates;
using ExampleWebApplication.Infraestruture.NHibernate;
using ExampleWebApplication.Models;
using NHibernate.Linq;
using Telerik.Web.Mvc;
using TelerikMvcGridCustomBindingHelper;
using TelerikMvcGridCustomBindingHelper.NHibernate;
using Order = ExampleWebApplication.Entities.Order;

namespace ExampleWebApplication.Controllers
{
    public class NHibernateController : Controller
    {
        public ActionResult Index()
        {
            return View();
        }

        #region Example #01: Custom ajax binding

        // Here we create our view, no need to pass any data yet.
        public ActionResult CustomAjaxBinding()
        {
            return View();
        }

        // And then send the data through ajax requests.
        [ApplyTransaction, GridAction(EnableCustomBinding = true)]
        public ActionResult CustomAjaxBindingData(GridCommand command)
        {
            // Telerik custom client binding demo: http://demos.telerik.com/aspnet-mvc/razor/grid/ajaxbinding

            // The same query will be used to count and list.
            // WARN: careful with Select N+1: http://nhprof.com/Learn/Alerts/SelectNPlusOne
            var query = NHibernateHelper.CurrentSession.Query<Product>().Fetch(x => x.Category);

            var gridHelper = new GridCustomBindingHelper<Product, ProductModel>(command, query);

            var gridModel = gridHelper.BuildGridModel();

            return View(gridModel);
        }

        #endregion

        #region Example #02: Custom server binding

        // We must enter the name of the grid, otherwise it will not work.
        [ApplyTransaction, GridAction(GridName = "Grid")]
        public ActionResult CustomServerBinding(GridCommand command)
        {
            // Telerik custom server binding demo: http://demos.telerik.com/aspnet-mvc/razor/grid/customserverbinding

            // We have to inform the pageSize here, since in the first call, 
            // the GridCommand is not correctly initialized by the grid, but 
            // dont forget to set it in the grid too.
            command.PageSize = 15; // Please put the same pageSize as is in the view

            var query = NHibernateHelper.CurrentSession.Query<Product>().Fetch(x => x.Category);

            var gridHelper = new GridCustomBindingHelper<Product, ProductModel>(command, query);

            var gridModel = gridHelper.BuildGridModel();

            ViewBag.Total = gridModel.Total;
            ViewBag.Data = gridModel.Data;

            return View();
        }

        #endregion

        #region Example #03: Two queries to count and list data

        public ActionResult TwoQueriesToCountAndList()
        {
            return View();
        }

        [ApplyTransaction, GridAction(EnableCustomBinding = true)]
        public ActionResult TwoQueriesToCountAndListData(GridCommand command)
        {
            var query = NHibernateHelper.CurrentSession.Query<Product>().Fetch(x => x.Category);
            var queryToCount = NHibernateHelper.CurrentSession.Query<Product>();

            var gridQueryProvider = new GridQueryProvider(query, queryToCount);

            var gridHelper = new GridCustomBindingHelper<Product, ProductModel>(command, gridQueryProvider);

            var gridModel = gridHelper.BuildGridModel();

            return View(gridModel);
        }

        #endregion

        #region Example #04: NHibernate QueryOver API

        public ActionResult NHibernateQueryOver()
        {
            return View();
        }

        [ApplyTransaction, GridAction(EnableCustomBinding = true)]
        public ActionResult NHibernateQueryOverData(GridCommand command)
        {
            // QueryOver API from NHibernate, fetching an association is not a problem here
            // WARN: careful with Select N+1: http://nhprof.com/Learn/Alerts/SelectNPlusOne
            var query = NHibernateHelper.CurrentSession.QueryOver<Product>().Fetch(x => x.Category).Eager;

            var gridHelper = new NHibernateGridCustomBindingHelper<Product, ProductModel>(command, query);

            var gridModel = gridHelper.BuildGridModel();

            return View(gridModel);
        }

        #endregion

        #region Example #05: Mapping techniques

        public ActionResult MappingTechniques()
        {
            return View();
        }

        [ApplyTransaction, GridAction(EnableCustomBinding = true)]
        public ActionResult MappingTechniquesData(GridCommand command)
        {
            var query = NHibernateHelper.CurrentSession.Query<Order>()
                .Fetch(order => order.Customer)
                .Fetch(order => order.Employee)
                .Fetch(order => order.Shipper);

            var gridHelper = new GridCustomBindingHelper<Order, OrderModel>(command, query);

            var gridModel = gridHelper.BuildGridModel();

            return View(gridModel);
        }

        #endregion

        #region Example #06: Aggregates server side [NOT SUPPORTED YET]

        // We must enter the name of the grid, otherwise it will not work.
        [ApplyTransaction, GridAction(GridName = "Grid")]
        public ActionResult AggregatesServerSide(GridCommand command)
        {
            // This functionality is NOT YET natively supported by the Telerik Grid:
            // Question 01: http://www.telerik.com/community/forums/aspnet-mvc/grid/aggregates-with-custom-binding.aspx
            // Question 02: http://www.telerik.com/community/forums/aspnet-mvc/grid/custom-binding-and-aggregates-groupfootertemplates.aspx

            // Telerik server aggregates demo: http://demos.telerik.com/aspnet-mvc/razor/grid/aggregatesserverside

            // Believe me, I have tried hard to add this feature, but unfortunately I could not get to it.
            // I assure you that I will continue on my journey to make it work, but the best chance is to wait for native support.
            // If you really need aggregates, try the ajax version as shown in the Example 07: Aggregates client side (Ajax).

            // We have to inform the pageSize here, since in the first call, 
            // the GridCommand is not correctly initialized by the grid, but 
            // dont forget to set it in the grid too.
            command.PageSize = 15; // Please put the same pageSize as is in the grid

            var query = NHibernateHelper.CurrentSession.Query<Product>().Fetch(x => x.Category);

            var gridHelper = new GridCustomBindingHelper<Product, ProductModel>(command, query);

            var gridModel = gridHelper.BuildGridModel();

            ViewBag.Total = gridModel.Total;
            ViewBag.Data = gridModel.Data;

            return View();
        }

        #endregion

        #region Example #07: Aggregates client side (Ajax)

        // Example #07.1: Generic linq
        public ActionResult AggregatesClientSide()
        {
            return View();
        }

        // Example #07.2: NHibernate QueryOver API
        public ActionResult AggregatesClientSideNHibernate()
        {
            return View();
        }

        // And then send the data through ajax requests.
        [ApplyTransaction, GridAction(EnableCustomBinding = true)]
        public ActionResult AggregatesClientSideData(GridCommand command)
        {
            // Telerik demo: http://demos.telerik.com/aspnet-mvc/razor/grid/aggregatesajax

            // This functionality (aggregates with custom binding) is NOT YET natively supported by the Telerik Grid:
            // Question 01: http://www.telerik.com/community/forums/aspnet-mvc/grid/aggregates-with-custom-binding.aspx
            // Question 02: http://www.telerik.com/community/forums/aspnet-mvc/grid/custom-binding-and-aggregates-groupfootertemplates.aspx

            var query = NHibernateHelper.CurrentSession.Query<Product>().Fetch(x => x.Category);

            var gridHelper = new GridCustomBindingHelper<Product, ProductModel>(command, query);

            var gridModel = gridHelper.BuildGridModel();

            return View(gridModel);
        }

        // And then send the data through ajax requests.
        [ApplyTransaction, GridAction(EnableCustomBinding = true)]
        public ActionResult AggregatesClientSideNHibernateData(GridCommand command)
        {
            // Telerik demo: http://demos.telerik.com/aspnet-mvc/razor/grid/aggregatesajax

            // This functionality (aggregates with custom binding) is NOT YET natively supported by the Telerik Grid:
            // Question 01: http://www.telerik.com/community/forums/aspnet-mvc/grid/aggregates-with-custom-binding.aspx
            // Question 02: http://www.telerik.com/community/forums/aspnet-mvc/grid/custom-binding-and-aggregates-groupfootertemplates.aspx

            // QueryOver API from NHibernate, fetching an association is not a problem here
            // Careful with Select N+1: http://nhprof.com/Learn/Alerts/SelectNPlusOne
            var query = NHibernateHelper.CurrentSession.QueryOver<Product>().Fetch(x => x.Category).Eager;

            var gridHelper = new NHibernateGridCustomBindingHelper<Product, ProductModel>(command, query);

            var gridModel = gridHelper.BuildGridModel();

            return View(gridModel);
        }

        #endregion

        #region Example #08: Direct usage of a TEntity (without TEntity to TViewModel mappings)

        public ActionResult DirectUsageOfTEntity()
        {
            return View();
        }

        public ActionResult DirectUsageOfTEntityNHibernate()
        {
            return View();
        }

        // Example #08.1: Generic linq
        [ApplyTransaction, GridAction(EnableCustomBinding = true)]
        public ActionResult DirectUsageOfTEntityData(GridCommand command)
        {
            var query = NHibernateHelper.CurrentSession.Query<Supplier>();

            var gridHelper = new GridCustomBindingHelper<Supplier>(command, query);

            var gridModel = gridHelper.BuildGridModel();

            return View(gridModel);
        }

        // Example #08.2: NHibernate QueryOver API
        [ApplyTransaction, GridAction(EnableCustomBinding = true)]
        public ActionResult DirectUsageOfTEntityNHibernateData(GridCommand command)
        {
            var query = NHibernateHelper.CurrentSession.QueryOver<Supplier>();

            var gridHelper = new NHibernateGridCustomBindingHelper<Supplier>(command, query);

            var gridModel = gridHelper.BuildGridModel();

            return View(gridModel);
        }

        #endregion

        #region Example #09: Custom aggregates functions - client side (Ajax)

        public ActionResult CustomAggregatesFunctionsClientSide()
        {
            return View();
        }

        public ActionResult CustomAggregatesFunctionsInlineClientSide()
        {
            return View();
        }

        public ActionResult CustomAggregatesFunctionsIgnoreGroupAggregatesClientSide()
        {
            return View();
        }

        public ActionResult CustomAggregatesFunctionsNHibernateClientSide()
        {
            return View();
        }

        public ActionResult CustomAggregatesFunctionsNHibernateInlineClientSide()
        {
            return View();
        }

        public ActionResult CustomAggregatesFunctionsNHibernateIgnoreGroupAggregatesClientSide()
        {
            return View();
        }

        public ActionResult CustomAggregatesFunctionsInlineDirectUsageOfTEntityClientSide()
        {
            return View();
        }

        // Example #09.1: Generic linq custom aggregates via an inherited class
        [ApplyTransaction, GridAction(EnableCustomBinding = true)]
        public ActionResult CustomAggregatesFunctionsClientSideData(GridCommand command)
        {
            var query = NHibernateHelper.CurrentSession.Query<Product>().Fetch(x => x.Category);

            var gridHelper = new GridCustomBindingHelper<Product, ProductModel>(command, query)
                .AddAggregateFunction<SumUnitPriceMultipliedByUnitsOnOrderAggregateFunction>(product => product.Units_OnOrder);

            var gridModel = gridHelper.BuildGridModel();

            return View(gridModel);
        }

        // Example #09.2: Generic linq custom aggregates via an inline function
        [ApplyTransaction, GridAction(EnableCustomBinding = true)]
        public ActionResult CustomAggregatesFunctionsInlineClientSideData(GridCommand command)
        {
            var query = NHibernateHelper.CurrentSession.Query<Product>().Fetch(x => x.Category);

            var gridHelper = new GridCustomBindingHelper<Product, ProductModel>(command, query)
                .AddAggregateFunction(product => product.Units_OnOrder, products => products.Sum(x => x.UnitPrice * x.Units_On_Order));

            var gridModel = gridHelper.BuildGridModel();

            return View(gridModel);
        }

        // Example #09.3: Generic linq custom aggregates via an inherited class and ignoring group aggregates
        [ApplyTransaction, GridAction(EnableCustomBinding = true)]
        public ActionResult CustomAggregatesFunctionsIgnoreGroupAggregatesClientSideData(GridCommand command)
        {
            var query = NHibernateHelper.CurrentSession.Query<Product>().Fetch(x => x.Category);

            var gridHelper = new GridCustomBindingHelper<Product, ProductModel>(command, query)
                //.AddAggregateFunction<SumUnitPriceMultipliedByUnitsOnOrderAggregateFunction>(product => product.Units_OnOrder)
                .AddAggregateFunction(product => product.Units_OnOrder, products => products.Sum(x => x.UnitPrice * x.Units_On_Order))
                .IgnoreGroupAggregates();

            var gridModel = gridHelper.BuildGridModel();

            return View(gridModel);
        }

        // Example #09.4: NHibernate QueryOver API custom aggregates via an inherited class
        [ApplyTransaction, GridAction(EnableCustomBinding = true)]
        public ActionResult CustomAggregatesFunctionsNHibernateClientSideData(GridCommand command)
        {
            var query = NHibernateHelper.CurrentSession.QueryOver<Product>().Fetch(x => x.Category).Eager;

            var gridHelper = new NHibernateGridCustomBindingHelper<Product, ProductModel>(command, query)
                //.AddAggregateFunction<SumUnitPriceMultipliedByUnitsOnOrderNHibernateAggregateFunction>(product => product.Units_OnOrder);
                //.AddAggregateFunction<SumUnitPriceMultipliedByUnitsOnOrderExpressionAggregateFunction>(product => product.Units_OnOrder);
                .AddAggregateFunction<SumUnitPriceMultipliedByUnitsOnOrderLinqAggregateFunction>(product => product.Units_OnOrder);

            var gridModel = gridHelper.BuildGridModel();

            return View(gridModel);
        }

        // Example #09.5: NHibernate QueryOver API custom aggregates via an inline function
        [ApplyTransaction, GridAction(EnableCustomBinding = true)]
        public ActionResult CustomAggregatesFunctionsNHibernateInlineClientSideData(GridCommand command)
        {
            var query = NHibernateHelper.CurrentSession.QueryOver<Product>().Fetch(x => x.Category).Eager;

            var gridHelper = new NHibernateGridCustomBindingHelper<Product, ProductModel>(command, query)
                //.AddAggregateFunction(product => product.Units_OnOrder,
                //    entities => entities.Aggregate(x => x.UnitPrice * x.Units_On_Order).Sum(),
                //    products => products.Sum(x => x.UnitPrice * x.Units_On_Order));
                .AddAggregateFunction(product => product.Units_OnOrder, products => products.Sum(x => x.UnitPrice * x.Units_On_Order));

            var gridModel = gridHelper.BuildGridModel();

            return View(gridModel);
        }

        // Example #09.6: NHibernate QueryOver API custom aggregates via an inline function and ignoring group aggregates
        [ApplyTransaction, GridAction(EnableCustomBinding = true)]
        public ActionResult CustomAggregatesFunctionsNHibernateIgnoreGroupAggregatesClientSideData(GridCommand command)
        {
            var query = NHibernateHelper.CurrentSession.QueryOver<Product>().Fetch(x => x.Category).Eager;

            var gridHelper = new NHibernateGridCustomBindingHelper<Product, ProductModel>(command, query)
                .AddAggregateFunction(product => product.Units_OnOrder, products => products.Sum(x => x.UnitPrice * x.Units_On_Order))
                .IgnoreGroupAggregates();

            var gridModel = gridHelper.BuildGridModel();

            return View(gridModel);
        }

        // Example #09.7: Generic linq custom aggregates via an inline function and direct usage of a TEntity (without TEntity to TViewModel mappings)
        [ApplyTransaction, GridAction(EnableCustomBinding = true)]
        public ActionResult CustomAggregatesFunctionsInlineDirectUsageOfTEntityClientSideData(GridCommand command)
        {
            var query = NHibernateHelper.CurrentSession.Query<Supplier>();

            var gridHelper = new GridCustomBindingHelper<Supplier>(command, query)
                .AddAggregateFunction(supplier => supplier.SupplierID, suppliers => suppliers.Sum(x => x.SupplierID));

            var gridModel = gridHelper.BuildGridModel();

            return View(gridModel);
        }

        #endregion

        #region Example #10: Using projections to return only the necessary columns from the database

        public ActionResult UsingProjections()
        {
            return View();
        }

        public ActionResult UsingProjectionsNHibernate()
        {
            return View();
        }

        // Example #10.1: Generic linq
        [ApplyTransaction, GridAction(EnableCustomBinding = true)]
        public ActionResult UsingProjectionsData(GridCommand command)
        {
            var query = NHibernateHelper.CurrentSession.Query<Product>();
            // with projections there's no need to fetch anything
            //.Fetch(x => x.Category)

            var gridHelper = new GridCustomBindingHelper<Product, ProductModel>(command, query)
                .UseProjections();
                //.UseProjections(options => options.Ignore(product => product.Discontinued));
                //.UseProjections(options => options.Include(product => product.Discontinued));
                //.UseProjections(options => options.Use(product => product.Discontinued));

            var gridModel = gridHelper.BuildGridModel();

            return View(gridModel);
        }

        // Example #10.2: NHibernate QueryOver API
        [ApplyTransaction, GridAction(EnableCustomBinding = true)]
        public ActionResult UsingProjectionsNHibernateData(GridCommand command)
        {
            var query = NHibernateHelper.CurrentSession.QueryOver<Product>();
            // with projections there's no need to fetch anything
            //.Fetch(x => x.Category).Eager;

            var gridHelper = new NHibernateGridCustomBindingHelper<Product, ProductModel>(command, query)
                .UseProjections();
                //.UseProjections(options => options.Ignore(product => product.Discontinued));
                //.UseProjections(options => options.Include(product => product.Discontinued));
                //.UseProjections(options => options.Use(product => product.Discontinued));

            var gridModel = gridHelper.BuildGridModel();

            return View(gridModel);
        }

        #endregion

        #region Example #11: BooleanSubstitutes (See the mappings on Global.asax)

        public ActionResult BooleanSubstitute()
        {
            return View();
        }

        public ActionResult BooleanSubstituteNHibernate()
        {
            return View();
        }

        // Example #11.1: Generic linq
        [ApplyTransaction, GridAction(EnableCustomBinding = true)]
        public ActionResult BooleanSubstituteData(GridCommand command)
        {
            // Add a BooleanSubstitute for a specifique property
            //GridModelMapper.CreateMap<Customer, CustomerModel>()
            //    .MapProperty(customer => customer.Bool, model => model.Bool)
            //    .WithBooleanSubstitute<BooleanSubstitute>();

            // Add a GlobalBooleanSubstitute for any boolean property
            //GridModelMapper.Configuration.SetGlobalBooleanSubstitute<GlobalBooleanSubstitute>();

            var query = NHibernateHelper.CurrentSession.Query<Customer>();

            var gridHelper = new GridCustomBindingHelper<Customer, CustomerModel>(command, query);

            var gridModel = gridHelper.BuildGridModel();

            return View(gridModel);
        }

        // Example #11.2: NHibernate QueryOver API
        [ApplyTransaction, GridAction(EnableCustomBinding = true)]
        public ActionResult NHibernateBooleanSubstituteData(GridCommand command)
        {
            // Add a BooleanSubstitute for a specifique property
            //GridModelMapper.CreateMap<Customer, CustomerModel>()
            //    .MapProperty(customer => customer.Bool, model => model.Bool)
            //    .WithBooleanSubstitute<BooleanSubstitute>();

            // Add a GlobalBooleanSubstitute for any boolean property
            //GridModelMapper.Configuration.SetGlobalBooleanSubstitute<GlobalBooleanSubstitute>();

            var query = NHibernateHelper.CurrentSession.QueryOver<Customer>();

            var gridHelper = new NHibernateGridCustomBindingHelper<Customer, CustomerModel>(command, query);

            var gridModel = gridHelper.BuildGridModel();

            return View(gridModel);
        }

        #endregion

        #region Example #12: ValueResolvers (See the mappings on Global.asax)

        public ActionResult ValueResolver()
        {
            return View();
        }

        public ActionResult ValueResolverNHibernate()
        {
            return View();
        }

        // Example #12.1: Generic linq
        [ApplyTransaction, GridAction(EnableCustomBinding = true)]
        public ActionResult ValueResolverData(GridCommand command)
        {
            // Add a DecimalValueResolver for a specifique property
            //GridModelMapper.CreateMap<Order, OrderModel>()
            //    .MapProperty(order => order.Freight, model => model.Freight)
            //    .WithResolver<DecimalValueResolver>();

            // Add a GlobalDecimalValueResolver for any property
            //GridModelMapper.Configuration.AddGlobalValueResolver<GlobalDecimalValueResolver>();

            var query = NHibernateHelper.CurrentSession.Query<Order>()
                .Fetch(order => order.Customer)
                .Fetch(order => order.Employee)
                .Fetch(order => order.Shipper);

            var gridHelper = new GridCustomBindingHelper<Order, OrderModel>(command, query).UseProjections(options => options.Ignore(order => order.Customer.EfBool));

            var gridModel = gridHelper.BuildGridModel();

            return View(gridModel);
        }

        // Example #12.2: NHibernate QueryOver API
        [ApplyTransaction, GridAction(EnableCustomBinding = true)]
        public ActionResult ValueResolverNHibernateData(GridCommand command)
        {
            // Add a DecimalValueResolver for a specifique property
            //GridModelMapper.CreateMap<Order, OrderModel>()
            //    .MapProperty(order => order.Freight, model => model.Freight)
            //    .WithResolver<DecimalValueResolver>();

            // Add a GlobalDecimalValueResolver for any property
            //GridModelMapper.Configuration.AddGlobalValueResolver<GlobalDecimalValueResolver>();

            var query = NHibernateHelper.CurrentSession.QueryOver<Order>()
                .Fetch(order => order.Customer).Eager
                .Fetch(order => order.Employee).Eager
                .Fetch(order => order.Shipper).Eager;

            var gridHelper = new NHibernateGridCustomBindingHelper<Order, OrderModel>(command, query).UseProjections(options => options.Ignore(order => order.Customer.EfBool));

            var gridModel = gridHelper.BuildGridModel();

            return View(gridModel);
        }

        #endregion

        #region Example #13: ORM integration (Especially with NHibernate, see the settings on Global.asax)

        public ActionResult ORMIntegration()
        {
            return View();
        }

        public ActionResult ORMIntegrationNHibernate()
        {
            return View();
        }

        // Example #13.1: ORM integration (Generic linq)
        [/*ApplyTransaction, */GridAction(EnableCustomBinding = true)]
        public ActionResult ORMIntegrationData(GridCommand command)
        {
            var gridHelper = new GridCustomBindingHelper<Product, ProductModel>(command)
                .UseProjections(); // Use projections to avoid "SelectNPlusOne"

            var gridModel = gridHelper.BuildGridModel();

            return View(gridModel);
        }

        // Example #13.2: ORM integration (NHibernate QueryOver API)
        [/*ApplyTransaction, */GridAction(EnableCustomBinding = true)]
        public ActionResult ORMIntegrationNHibernateData(GridCommand command)
        {
            var gridHelper = new NHibernateGridCustomBindingHelper<Product, ProductModel>(command)
                .UseProjections(); // Use projections to avoid "SelectNPlusOne"

            var gridModel = gridHelper.BuildGridModel();

            return View(gridModel);
        }

        #endregion

        #region Example #14: Filtering on nullable data

        public ActionResult FilteringNullableValues()
        {
            return View();
        }

        public ActionResult FilteringNullableValuesNHibernate()
        {
            return View();
        }

        // Example #14.1: Generic linq
        [ApplyTransaction, GridAction(EnableCustomBinding = true)]
        public ActionResult FilteringNullableValuesData(GridCommand command)
        {
            var query = NHibernateHelper.CurrentSession.Query<Order>();

            // Can be set globally for all entities
            //GridModelMapper.Configuration.AcceptNullValuesWhenFiltering();

            // Can be set globally for a specifique entity
            //GridModelMapper.Configuration.AcceptNullValuesWhenFiltering<Product>();

            // Can be set for a specifique property
            //GridModelMapper.CreateMap<Order, OrderModel>()
            //    //.MapProperty(order => order.ShipRegion, model => model.ShipRegion).WithOptions(options => options.AcceptNullValuesWhenFiltering())
            //    .MapProperty(order => order.ShipRegion, model => model.ShipRegion, options => options.AcceptNullValuesWhenFiltering());

            var gridHelper = new GridCustomBindingHelper<Order, OrderModel>(command, query)
                // Can be set for a specifique query
                .AcceptNullValuesWhenFiltering()
                // Let's use projections here to avoid Select+1.
                .UseProjections(options => options.Use(order => new { order.OrderID, order.ShipRegion }));
                //.UseProjections(options => options.Ignore(order => order.Customer.EfBool));

            var gridModel = gridHelper.BuildGridModel();

            return View(gridModel);
        }

        // Example #14.2: NHibernate QueryOver API
        [ApplyTransaction, GridAction(EnableCustomBinding = true)]
        public ActionResult FilteringNullableValuesNHibernateData(GridCommand command)
        {
            var query = NHibernateHelper.CurrentSession.QueryOver<Order>();

            // Can be set globally for all entities
            //GridModelMapper.Configuration.AcceptNullValuesWhenFiltering();

            // Can be set globally for a specifique entity
            //GridModelMapper.Configuration.AcceptNullValuesWhenFiltering<Product>();

            // Can be set for a specifique property
            //GridModelMapper.CreateMap<Order, OrderModel>()
            //    //.MapProperty(order => order.ShipRegion, model => model.ShipRegion).WithOptions(options => options.AcceptNullValuesWhenFiltering())
            //    .MapProperty(order => order.ShipRegion, model => model.ShipRegion, options => options.AcceptNullValuesWhenFiltering());

            var gridHelper = new NHibernateGridCustomBindingHelper<Order, OrderModel>(command, query)
                // Can be set for a specifique query
                .AcceptNullValuesWhenFiltering()
                // Let's use projections here to avoid Select+1.
                .UseProjections(options => options.Use(order => new { order.OrderID, order.ShipRegion }));
                //.UseProjections(options => options.Ignore(order => order.Customer.EfBool));

            var gridModel = gridHelper.BuildGridModel();

            return View(gridModel);
        }

        public ActionResult FilteringWithNullAliases()
        {
            return View();
        }

        public ActionResult FilteringWithNullAliasesNHibernate()
        {
            return View();
        }

        // Example #14.3: Generic linq
        [ApplyTransaction, GridAction(EnableCustomBinding = true)]
        public ActionResult FilteringWithNullAliasesData(GridCommand command)
        {
            // Try searching the column ShipRegion by "none" or "empty", is expected to return null values​​.
            var query = NHibernateHelper.CurrentSession.Query<Order>();

            // Can be set globally for all properties
            //GridModelMapper.Configuration.SetGlobalNullAlias<NullAlias>();

            // Can be set for a specifique property
            //GridModelMapper.CreateMap<Order, OrderModel>()
            //.MapProperty(order => order.ShipRegion, model => model.ShipRegion).WithOptions(options => options.WithNullAlias<ShipRegionNullAlias>())
                //.MapProperty(order => order.ShipRegion, model => model.ShipRegion, options => options.WithNullAlias<ShipRegionNullAlias>());
            
            var gridHelper = new GridCustomBindingHelper<Order, OrderModel>(command, query)
                // Let's use projections here to avoid Select+1.
                .UseProjections(options => options.Use(order => new { order.OrderID, order.ShipRegion }));
                //.UseProjections(options => options.Ignore(order => order.Customer.EfBool));

            var gridModel = gridHelper.BuildGridModel();

            return View(gridModel);
        }

        // Example #14.4: Generic linq
        [ApplyTransaction, GridAction(EnableCustomBinding = true)]
        public ActionResult FilteringWithNullAliasesNHibernateData(GridCommand command)
        {
            // Try searching the column ShipRegion by "none" or "empty", is expected to return null values​​.
            var query = NHibernateHelper.CurrentSession.QueryOver<Order>();

            // Can be set globally for all properties
            //GridModelMapper.Configuration.SetGlobalNullAlias<NullAlias>();

            // Can be set for a specifique property
            //GridModelMapper.CreateMap<Order, OrderModel>()
            //.MapProperty(order => order.ShipRegion, model => model.ShipRegion).WithOptions(options => options.WithNullAlias<ShipRegionNullAlias>())
                //.MapProperty(order => order.ShipRegion, model => model.ShipRegion, options => options.WithNullAlias<ShipRegionNullAlias>());

            var gridHelper = new NHibernateGridCustomBindingHelper<Order, OrderModel>(command, query)
                // Let's use projections here to avoid Select+1.
                .UseProjections(options => options.Use(order => new { order.OrderID, order.ShipRegion }));
                //.UseProjections(options => options.Ignore(order => order.Customer.EfBool));

            var gridModel = gridHelper.BuildGridModel();

            return View(gridModel);
        }

        public ActionResult FilteringWithNullSubstitutes()
        {
            return View();
        }

        public ActionResult FilteringWithNullSubstitutesNHibernate()
        {
            return View();
        }

        // Example #14.5: Generic linq
        [ApplyTransaction, GridAction(EnableCustomBinding = true)]
        public ActionResult FilteringWithNullSubstitutesData(GridCommand command)
        {
            var query = NHibernateHelper.CurrentSession.Query<Employee>();

            // Can be set globally for all properties
            //GridModelMapper.Configuration.SetGlobalNullSubstitute<EmployeeRegionNullSubstitute>();

            // Can be set for a specifique property
            //GridModelMapper.CreateMap<Employee, EmployeeModel>()
            //    .MapProperty(employee => employee.Region, model => model.Region, options => options.WithNullSubstitute<EmployeeRegionNullSubstitute>());

            var gridHelper = new GridCustomBindingHelper<Employee, EmployeeModel>(command, query)
                .AcceptNullValuesWhenFiltering() // { string.Empty, null } => Aliases that will be treated as null values and will be sent to the database within queries.
                .UseProjections(options => options.Use(employee => new { employee.EmployeeID, employee.FirstName, employee.LastName, employee.Region }));

            var gridModel = gridHelper.BuildGridModel();

            return View(gridModel);
        }

        // Example #14.6: Generic linq
        [ApplyTransaction, GridAction(EnableCustomBinding = true)]
        public ActionResult FilteringWithNullSubstitutesNHibernateData(GridCommand command)
        {
            var query = NHibernateHelper.CurrentSession.QueryOver<Employee>();

            // Can be set globally for all properties
            //GridModelMapper.Configuration.SetGlobalNullSubstitute<EmployeeRegionNullSubstitute>();

            // Can be set for a specifique property
            //GridModelMapper.CreateMap<Employee, EmployeeModel>()
            //    .MapProperty(employee => employee.Region, model => model.Region, options => options.WithNullSubstitute<EmployeeRegionNullSubstitute>());

            var gridHelper = new NHibernateGridCustomBindingHelper<Employee, EmployeeModel>(command, query)
                .AcceptNullValuesWhenFiltering() // { string.Empty, null } => Aliases that will be treated as null values and will be sent to the database within queries.
                .UseProjections(options => options.Use(employee => new { employee.EmployeeID, employee.FirstName, employee.LastName, employee.Region }));

            var gridModel = gridHelper.BuildGridModel();

            return View(gridModel);
        }

        #endregion
    }
}
